#top_toolbar_div .glyphicon {
    color: #333;
    cursor: default;
    top: 0;
    border-radius: 0;
}

#top_toolbar_div .selected {
    background: #ccc;
}


#top_toolbar_div .glyphicon:hover {
    color: #eee;
    background: #666;
}

#top_toolbar_div .deactivated .glyphicon {
    color: #ccc;
}

#top_toolbar_div .deactivated .glyphicon:hover {
    color: #ccc;
    background: none;
}

#top_toolbar_div .reversed {
    -moz-transform: scaleX(-1);
    -o-transform: scaleX(-1);
    -webkit-transform: scaleX(-1);
    transform: scaleX(-1);
    filter: FlipH;
    -ms-filter: "FlipH";
}

#top_toolbar_div {
    position: relative;
    flex: 0 0 auto;
    -webkit-flex: 0 0 auto;
    width: 100%;
    z-index: 99999;
}

#top_toolbar_div .nav {
    background: #eee;
    box-shadow: 0px 2px 4px 0px #999;
}

#filename{
    padding: 0.5em;
    cursor: default !important;
}

#Forum_link {
    padding: 0.5em;
    cursor: hand !important;
}

#filename:hover, #Forum_link:hover {
    color: #eee;
    background: #666;
}

#Help_button, #Forum_link{
    float: right;
}

#Toggle_ace, #Save_file, #Sync_editor_button, #Font_increase, #Font_decrease{
    float: right;
}

#top_toolbar_div li.disabled a.glyphicon {
    color: #ccc;
    background: #eee;
}

#top_toolbar_div li.in-progress a.glyphicon {
    color: #ccc;
    background: #c22;
}

#global_config_menu {
    background: purple;
    height: 20em;
    width: 20em;
    z-index: 99999;
    margin-top: 3em;
    position: absolute;
}
